@@ -581,25 +581,30 @@ namespace Konclude {
 					QString languageValueString;
 					if (parseLanguageString) {
 						cint64 lastLanguageSep = valueString.lastIndexOf("@");
-						if (lastLanguageSep > -1) {
+						if (lastLanguageSep > 0) {
 							stringValueString = valueString.mid(0,lastLanguageSep);
-							languageValueString = valueString.mid(lastLanguageSep+1);
-							bool validLanguageTag = true;
-							bool firstBindOccurred = false;
-							for (cint64 i = 0; i < languageValueString.length() && validLanguageTag; ++i) {
-								QChar ch = languageValueString.at(i);
-								if (!firstBindOccurred && i > 3) {
-									validLanguageTag = false;
+							if (!stringValueString.trimmed().isEmpty()) {
+								languageValueString = valueString.mid(lastLanguageSep + 1);
+								bool validLanguageTag = true;
+								bool firstBindOccurred = false;
+								for (cint64 i = 0; i < languageValueString.length() && validLanguageTag; ++i) {
+									QChar ch = languageValueString.at(i);
+									if (!firstBindOccurred && i > 3) {
+										validLanguageTag = false;
+									}
+									if (ch == QChar('-')) {
+										firstBindOccurred = true;
+									}
+									if (!ch.isLetterOrNumber() && ch != QChar('-')) {
+										validLanguageTag = false;
+									}
 								}
-								if (ch == QChar('-')) {
-									firstBindOccurred = true;
-								}
-								if (!ch.isLetterOrNumber() && ch != QChar('-')) {
-									validLanguageTag = false;
-								}
-							}
 
-							if (!validLanguageTag || true) {
+								if (!validLanguageTag) {
+									stringValueString = valueString;
+									languageValueString = "";
+								}
+							} else {
 								stringValueString = valueString;
 								languageValueString = "";
 							}
